* Generate unique plan identifier
gen plan = plnpbprc + plncntrc  

* Drop if claim is not covered
drop if drcvstcd!="C"

* Generating spending that occurs in each phase to estimate cost sharing
	* DEDUCTIBLE
	gen ded_totcost = 0
	gen ded_oopspend = 0
	replace ded_totcost = totalcst if bnftphas == "DD"
	replace ded_oopspend = ptpayamt if bnftphas == "DD"
	* PRE-ICL
	gen preicl_totcost = 0
	gen preicl_oopspend = 0
	replace preicl_totcost = totalcst if bnftphas == "PP" 
	replace preicl_oopspend = ptpayamt if bnftphas == "PP"
	* GAP 
	gen gap_totcost = 0
	gen gap_oopspend = 0
	replace gap_totcost = totalcst if bnftphas == "II" 
	replace gap_oopspend = ptpayamt if bnftphas == "II" 
	* CATASTROPHIC
	gen cat_totcost = 0
	gen cat_oopspend = 0 
	replace cat_totcost = totalcst if bnftphas == "CC"
	replace cat_oopspend = ptpayamt if bnftphas == "CC"

* To obtain a total number of claims in the year
gen count = 1 

* Making sure no observations are missing service dates
replace srvc_dt = td(01jan1960) if missing(srvc_dt)

* Merge in external drug classification data
merge m:1 prdsrvid using mergefiles/FDB/DrugClassifications, gen(_merge_drug)
drop if _merge_drug==1

* Merge in the month of the year the beneficiary joins Part D (from beneficiary file)
merge m:1 bene_id using ${int}/allage_joinmonth_`year', keep(match master) gen(_merge_jm)
gen partd_joinday = mdy(partd_joinmonth, 1, `year')

* Generating total spending, date of last claim, and January spending for each drug type
	* Classifying claims
	gen clm_any = 1
	gen clm_acute = (chronic_drug==0)
	gen clm_chr = (chronic_drug==1)
	gen clm_gen = (fdb_generic==1)
	gen clm_brand = (fdb_generic==0)
	gen clm_maint = (maint==1)
	gen clm_nonmaint = (maint==0)
	gen clm_hedis = (hedis_ind==1)
	gen clm_nonhedis = (hedis_ind==0)
	* Generating variables for total spending, date of last claim, and January spending
	foreach c in any acute chr gen brand maint nonmaint hedis nonhedis {
		gen total_`c' = totalcst if clm_`c'
			replace total_`c' = 0 if total_`c'==.
		gen date_`c' = srvc_dt if clm_`c'
			replace date_`c'=td(01jan1960) if date_`c'==.
		gen jan_total_`c' = totalcst if month(srvc_dt)==1 & clm_`c'
			replace jan_total_`c' = 0 if missing(jan_total_`c')
	}
	* Generating February spending for select drug types
	foreach c in any acute chr {
		gen feb_total_`c' = totalcst if month(srvc_dt)==1 & clm_`c'
			replace feb_total_`c' = 0 if missing(feb_total_`c')
	}

* Generating initial utilization variables for age 65 join month analysis
gen initial_total_any = totalcst if month(srvc_dt)<(partd_joinmonth+3)
	replace initial_total_any = 0 if initial_total_any==.
gen days_to_firstclaim_any = date_any-partd_joinday+1
	replace days_to_firstclaim_any = 92 if (days_to_firstclaim_any>92|days_to_firstclaim_any<1)

* Generating total spending and date of last claim for each therapeutic class (GTC)
foreach c in 2 11 14 20 23 32 35 38 41 56 59 62 65 68 71 80 86 89 {
	gen total_gtc`c' = totalcst if gtc==`c'
		replace total_gtc`c' = 0 if total_gtc`c'==.
	gen date_gtc`c' = srvc_dt if gtc==`c'
		replace date_gtc`c'=td(01jan1960) if date_gtc`c'==.
}

* Generating total spending by month
forv m = 1/12 {
	gen total_mo`m' = totalcst if month(srvc_dt)==`m'
		replace total_mo`m'=0 if total_mo`m'==.
}

* Coding final phase numerically to ensure it collapses correctly
gen finphas = 0
replace finphas = 1 if bnftphas=="XX"
replace finphas = 2 if bnftphas=="NA"
replace finphas = 3 if bnftphas=="DD"
replace finphas = 4 if bnftphas=="DP"
replace finphas = 5 if bnftphas=="PP"
replace finphas = 6 if bnftphas=="DI"
replace finphas = 7 if bnftphas=="PI"
replace finphas = 8 if bnftphas=="II"
replace finphas = 9 if bnftphas=="DC"
replace finphas = 10 if bnftphas=="PC"
replace finphas = 11 if bnftphas=="IC"
replace finphas = 12 if bnftphas=="CC"

* Collapse the data to the beneficiary-plan-year level
collapse ///
(sum) count oopspend=ptpayamt totcost=totalcst initial_* total_* clm_* ///
	preicl_totcost preicl_oopspend gap_totcost gap_oopspend ded_totcost ded_oopspend ///
	cat_totcost cat_oopspend jan_total_* feb_total_* lics_amt plro_amt othtroop ///
(max) date_* finalphase=finphas ///
(min) days_*, ///
by(plan bene_id) fast

* Recoding final phase variable to string
gen finphas = ""
replace finphas = "XX" if finalphase==1
replace finphas = "NA" if finalphase==2
replace finphas = "DD" if finalphase==3
replace finphas = "DP" if finalphase==4
replace finphas = "PP" if finalphase==5
replace finphas = "DI" if finalphase==6
replace finphas = "PI" if finalphase==7
replace finphas = "II" if finalphase==8
replace finphas = "DC" if finalphase==9
replace finphas = "PC" if finalphase==10
replace finphas = "IC" if finalphase==11
replace finphas = "CC" if finalphase==12

format date_* %td 
