cap log close
set more off
clear all

* Set current directory
cd /disk/agedisk3/medicare.work/finkels-DUA22559/kluender/

global date 070915 // set date
global int /disk/agedisk3/medicare.work/finkels-DUA22559/kluender/mergefiles/intermediatefiles_check // set location for intermediate files
global out /disk/agedisk3/medicare.work/finkels-DUA22559/kluender/mergefiles/finaldata // set location for final data output

scalar beneficiary = 1 // clean and append beneficiary files if == 1
scalar claim = 1 // clean and append claim data if == 1
scalar plan = 1 // clean plan data if plan == 1
scalar merge = 1 // merge beneficiary, claim, and plan data if == 1
scalar append = 1 // join all years if == 1
scalar baseline_restrictions = 1 // generate baseline sample

log using ${out}/datapreparation_${date}.log, replace

*****************************************************
*** STEP 1: Clean and append raw beneficiary data ***
*****************************************************
if beneficiary == 1 {

local num_files_2006 4
local num_files_2007 4
local num_files_2008 4
local num_files_2009 100
local file_path_2006 /disk/agedisk5/medicare/afinkels-DUA22559/medipd/data/20pct/ben/2006/ben20
local file_path_2007 /disk/agedisk5/medicare/afinkels-DUA22559/medipd/data/20pct/ben/2007/ben20
local file_path_2008 /disk/agedisk5/medicare/afinkels-DUA22559/medipd/data/20pct/ben/2008/ben20
local file_path_2009 /disk/agedisk5/medicare/afinkels-DUA22559/medipd/data/20pct/bsf/2009/bsf

forv year = 2006/2009 {
* Data is broken up into a number of files per year
* Cycles through each of these files:
forvalues x = 1/`num_files_`year'' {

	* Load the beneficiary data, includes Parts A, B, C, and D so beneficiaries not enrolled 
	* in Part D are dropped in the next step
	use `file_path_`year''_`year'_`x', clear

	* Generate a variable for months enrolled in Part D
	destring plncovmo, generate(partd) 
	* Drop beneficiaries not enrolled in Part D
	drop if partd == 0
	
	* Generate more useful indicators
	destring sex, replace
		drop if sex == 0
		gen female = sex==2
	destring race, replace
		gen white = race==1
	gen death = 1
		replace death = 0 if death_dt == .

	* Generate a monthly plan ID
	foreach n in 01 02 03 04 05 06 07 08 09 10 11 12 {
		gen plan`n' = pbpid`n'+cntrct`n'
		gen _cntrct`n' = substr(cntrct`n',1,1)
	}
	
	* Generate a monthly switch dummy to identify beneficiaries who switch plans from one month to the next
	local switchmo "0102 0203 0304 0405 0506 0607 0708 0809 0910 1011 1112"
	foreach mo of local switchmo {
	 	local m=substr("`mo'",1,2)
	 	local n=substr("`mo'",3,4)
		gen switch`n' = (plan`m' != plan`n' & /// different plan id
			!(missing(plan`m') | _cntrct`m'=="0" | _cntrct`m'=="N") & /// enrolled in a plan last month
			!(missing(plan`n') | _cntrct`n'=="0" | _cntrct`n'=="N")) // enrolled in a plan this month
	}	
	
	* Sum total number of switches per year
	gen numswitch = switch02 + switch03 + switch04 + switch05 + switch06 + switch07 + switch08 + switch09 + switch10 + switch11 + switch12
	
	* Generate indicators for enrollment in Parts A and B
	assert a_mo_cnt!=.
	assert b_mo_cnt!=.
	gen parta=a_mo_cnt>0 & a_mo_cnt!=.
	gen partb=b_mo_cnt>0 & b_mo_cnt!=.

	* Generate a variables for enrollment and cost sharing
	local i= 1
	foreach y in "01" "02" "03" "04" "05" "06" "07" "08" "09" "10" "11" "12" {
		replace cstshr`y'="14" if cstshr`y'=="XX"
		gen temp=real(cstshr`y')
		gen partd_mo_`i'=`i' if temp<=9 & temp>0
		drop temp
		local i=`i'+1
	}

	* Pull beneficiaries' birth and join months
	gen birthmonth=month(bene_dob)
	egen partd_joinmonth=rowmin(partd_mo*)
	drop partd_mo_*

	* Generate a dummy for whether a beneficiary receives any cost sharing
	gen no_costsharing = 1
		foreach n in 01 02 03 04 05 06 07 08 09 10 11 12 {
			replace no_costsharing = 0 if cstshr`n'!="09" & partd_joinmonth<=`n'
	}

	keep bene_id bene_zip state_cd sex race white female age dual_mo death numswitch crdcovsw orec ///
	plan12 parta partb partd partd_joinmonth birthmonth bene_dob no_costsharing strctflg

	tempfile ben_`year'_`x'
	save `ben_`year'_`x''

}

* Append files to create a complete file for each year
use `ben_`year'_1', clear
	forvalues x = 2/`num_files_`year'' {
		append using `ben_`year'_`x'', force
	}
save ${int}/benmerge`year', replace

keep bene_id partd_joinmonth
save ${int}/allage_joinmonth_`year', replace

} // end foreach

} // end beneficiary 

*****************************************************
****** STEP 2: Clean and append raw claim data ******
*****************************************************
if claim == 1 {

forv year = 2006/2009 {

	if `year'==2006|`year'==2007|`year'==2008 {
		forvalues x = 1/4 {
		forvalues y = 1/100 {	
			di "CLAIMS FILE YEAR: `year' X: `x' Y: `y'"
			* Load the claims subfile (400 in 2007 and 2008, 100 in 2009)
			use /disk/agedisk5/medicare/afinkels-DUA22559/medipd/data/20pct/pde/`year'/100files/stata/pde20_`year'_`x'_`y', clear
			* Run the claims preparation code
			qui: include Programs/FinalCode/DataPreparation_SubProgram1.do
			* Save intermediate collapsed claims file
			tempfile claims_`year'_`x'_`y'
			save `claims_`year'_`x'_`y'', replace
		}
		}

		* Append all the collapsed subfiles together to create one file for each year
		use `claims_`year'_1_1', clear
			drop if !missing(bene_id)
			forv x = 1/4 {
			forv y = 1/100 {
				append using `claims_`year'_`x'_`y''
			}
			}
		save ${int}/claimsmerge`year', replace
	}

	if `year'==2009 {
		forv y = 1/100 {
			di "CLAIMS FILE YEAR: `year' Y: `y'"
			* Load the claims subfile (400 in 2007 and 2008, 100 in 2009)
			use /disk/agedisk5/medicare/afinkels-DUA22559/medipd/data/20pct/pde/2009/100files/pde20_2009_`y', clear
			* Run the claims preparation code
			qui: include Programs/FinalCode/DataPreparation_SubProgram1.do
			* Save intermediate collapsed claims file
			tempfile claims_`year'_`y'
			save `claims_`year'_`y''
		} // end y

		* Append files to create a complete file for 2009
		use `claims_`year'_1', clear
			forv y = 2/100 {
				append using `claims_`year'_`y''
			}
		save ${int}/claimsmerge`year', replace
	} // end if year==2009
	
} // end foreach year
} // end claim

*****************************************************
********* STEP 3: Clean raw plan data ***************
*****************************************************
if plan == 1 {

forv year = 2006/2009 {

	* Load the plan files for each year
	use /disk/agedisk5/medicare/afinkels-DUA22559/medipd/data/20pct/pln/`year'/pln20_`year'_1, clear

	* Generate indicator for no gap coverage (different coding in 2008)
	if `year'==2006|`year'==2007|`year'==2009 {
	gen nogapcov = (gapcovtp=="N"|gapcovtp=="9")
	}
	if `year' == 2008 {
	gen nogapcov = (gapcovtp=="N")
	}
	* Generate indicator for no gap
	gen nogap = missing(icl_amt)

	* Generate our unique plan ID
	gen plan = plan_id + cntrctid

	* Rename copays and coinsurance for pre-ICL and gap
	forv tier = 1/3 {
		if `year'==2006 {
			rename pipp1p0`tier' pip1p0`tier'
			rename pipp1n0`tier' pip1n0`tier'
		}
		rename pip1n0`tier' t`tier'coin
		rename pip1p0`tier' t`tier'copay
		if `year'!=2006 { 
			rename gip1n0`tier' gt`tier'coin
			rename gip1p0`tier' gt`tier'copay
		}
	} 

	if `year'!=2006 {
		keep plan ded_amt icl_amt t1coin t1copay t2coin t2copay t3coin t3copay egwp_ind ///
		gt1coin gt1copay gt2coin gt2copay gt3coin gt3copay nogap drgbentp ///
		nogapcov oopt_amt pst_oopt pln_type
	}
	if `year'==2006 {
		keep plan ded_amt icl_amt t1coin t1copay t2coin t2copay t3coin t3copay egwp_ind ///
		nogap drgbentp nogapcov oopt_amt pst_oopt pln_type
	}

	save ${int}/plnmerge`year', replace

} // end foreach year
} // end plan

**************************************************************
*** STEP 4: Merge based on individual and plan identifiers ***
**************************************************************
if merge == 1 {

forv year = 2006/2009 {

	* Load claims data for the given year
	use ${int}/claimsmerge`year', clear

	* Merge claims data (master) to plan data (using)
	merge m:1 plan using ${int}/plnmerge`year', gen(_merge_pln)

	* Merge claims/plan data (master) to beneficiary data (using)
	merge m:1 bene_id using ${int}/benmerge`year', gen(_merge_ben)

	* We observe 5 merge combinations and drop all but 2:
	* KEEP: 
		* 1. Matched Beneficiaries, Claims, and Plans
		* 2. Matched Beneficiaries and Plans with No Claims (need to recode variables first):
		
			* We need to recode the missing variables to zeroes (for spending and date variables)
			* and the maximum number of days (for censored timing variables)
			* Set locals for the variables we need to recode
			local spendvar count oopspend totcost initial_* preicl_* gap_* cat_* ded_oopspend ///
				ded_totcost total_* jan_total_* feb_total_*
			local days days_to_*
			local dates date_*
			local planvar_2006 "ded_amt icl_amt t1coin t1copay t2coin t2copay t3coin t3copay nogap nogapcov oopt_amt pst_oopt pln_type drgbentp"
			local planvar_2007 "ded_amt icl_amt t1coin t1copay t2coin t2copay t3coin t3copay gt1coin gt1copay gt2coin gt2copay gt3coin gt3copay nogap nogapcov oopt_amt pst_oopt pln_type drgbentp"
			local planvar_2008 "ded_amt icl_amt t1coin t1copay t2coin t2copay t3coin t3copay gt1coin gt1copay gt2coin gt2copay gt3coin gt3copay nogap nogapcov oopt_amt pst_oopt pln_type drgbentp"
			local planvar_2009 "ded_amt icl_amt t1coin t1copay t2coin t2copay t3coin t3copay gt1coin gt1copay gt2coin gt2copay gt3coin gt3copay nogap nogapcov oopt_amt pst_oopt pln_type drgbentp"

			preserve
				keep if _merge_ben == 2 // make subset of data with just unmerged
				rename plan plan_temp
				rename plan12 plan //use plan12 (from BEN dataset) to merge with plan cahracteristics
				drop `planvar_`year''
				merge m:1 plan using ${int}/plnmerge`year', keep(match)

				* The observations we keep are beneficiaries with valid plan IDs who do have not filed claims,
				* so we recode them as 0 in spending variables
				foreach var of varlist `spendvar' {
					replace `var'=0
				}
				foreach day of varlist `days' {
					replace `day'=92
				}		
				foreach date of varlist `dates' {
					replace `date'=td(01jan1960)
				}
				replace finphas = "PP" if (ded_amt==0 | missing(ded_amt))
				replace finphas = "DD" if ded_amt>0 & !missing(ded_amt)
				
				gen plan12 = plan
				drop plan_temp _merge
				tempfile beneswithnoclaims
				save `beneswithnoclaims', replace
			restore
			drop if _merge_ben == 2 // after restore, drop all unmatched
			append using `beneswithnoclaims', force // append to replace the drops with merged datapoints

	* DROP: 
		* 3. Claims not matched to beneficiaries
		drop if _merge_ben==1
		* 4. Claims not matched to a plan
		drop if _merge_pln==1
		* 5. Plans not matched to any claims
		drop if _merge_pln==2
		
	* Make first sample selection drops
	* 1. Drop beneficiaries not in the strict 20% Medicare sample
	keep if strctflg=="05"|strctflg=="15"
	* 2. Drop beneficiaries not in the United States
	destring state_cd, gen(statenum)
	drop if statenum == 40 | statenum == 48 | statenum >=54 

	* We now have a dataset at beneficiary-plan level for each year
	save ${int}/bene`year', replace

} // end foreach

} // end merge

********************************************************
**** STEP 5: Append years to make multiyear dataset ****
********************************************************
if append == 1 {

* This section just appends the datasets from each of the three years

* The beneficiary IDs change from 2008-2009 so we convert the 2009 ID's to the 2007/2008 ones
use ${int}/bene2009, clear
	merge m:1 bene_id using /disk/agedisk5/medicare/afinkels-DUA22559/medipd/data/20pct/ben/xwalk20174_16702, gen(_dua_bene)
	drop if _dua_bene==2
	replace bene_id20174=bene_id if bene_id20174==""
	drop bene_id
	rename bene_id20174 bene_id
save ${int}/bene2009_mod, replace

* Now append the three years
use ${int}/bene2007, clear
	append using ${int}/bene2008 ${int}/bene2009_mod, gen(yr)
	replace yr = yr+2007

***********************
* SAMPLE RESTRICTIONS *
***********************

* Dropping beneficiairies who join for a reason other than turning 65
drop if age<65
keep if orec=="0"
count

* Dropping beneficiairies who are eligible for Medicaid as well
encode dual_mo, gen(dual)
drop if dual > 1 & dual != . // drop duals
count

* Keeping only beneficiaries with no listed 3rd party cost sharing
keep if no_costsharing==1
keep if crdcovsw=="0"
count

* Keeping only stand-alone Medicare Part D plans
keep if pln_type=="29"
count

* Dropping beneficiaries who switch plans within the year
drop if numswitch!=0 
count

* Dropping beneficiaries who die in the year
drop if death == 1
count

* Dropping any remaining duplicate beneficiaries in the year
sort bene_id yr plan 
by bene_id yr: gen dup = _N
drop if dup!=1
drop dup
count

* Save the full data
save ${int}/bene070809, replace

}

*****************************************************
**** STEP 6A: Generate baseline analysis dataset ****
*****************************************************
if baseline_restrictions == 1 {

use ${int}/bene070809, clear
count

* Generating a variable describing gap coverage
gen plan_type=""
replace plan_type = "nogapcov" if nogapcov==1
replace plan_type = "somegapcov" if nogapcov==0
replace plan_type = "no_icl" if icl_amt==.
tab plan_type, mi

* Generating a variable for the standard ICL/Ded amount in each year
gen standardicl = 2400 if yr==2007
replace standardicl = 2510 if yr==2008
replace standardicl=2700 if yr==2009
gen has_standardicl= icl_amt==standardicl
gen standardded = 265 if yr==2007
replace standardded = 275 if yr==2008
replace standardded = 295 if yr==2009
gen has_standardded = ded_amt==standardded

* Generating a variable for the type of plan (deductible/no deductible, gap coverage/no gap coverage)
gen plan_group=""
replace plan_group = "ded_nogap" if ded_amt>0 & ded_amt!=. & nogapcov==1
replace plan_group = "noded_nogap" if ded_amt==0 & nogapcov==1
replace plan_group = "noded_gapcov" if ded_amt==0 & nogapcov==0
replace plan_group = "ded_gapcov" if ded_amt>0 & ded_amt!=. & nogapcov==0
replace plan_group = "no_icl" if icl_amt==.

* Generate cleaner version of final phase variable that eliminates the straddle codes and orders the phases
gen finphas_clean = ""
replace finphas_clean = "A_Ded" if finphas=="DD" // Deductible phase
replace finphas_clean = "B_PreICL" if finphas=="DP"|finphas=="PP" // Pre-ICL phase
replace finphas_clean = "C_ICL" if finphas=="DI"|finphas=="PI"|finphas=="II" // ICL Phase
replace finphas_clean = "D_Cat" if finphas=="DC"|finphas=="PC"|finphas=="IC"|finphas=="CC" // Catastrophic Phase
replace finphas_clean = "NA" if finphas=="NA"

* Generate variables for difference between spending and various kink points
gen icl_diff = totcost-icl_amt
replace icl_diff = totcost-standardicl if icl_amt==.
gen ded_diff = totcost-ded_amt
gen cat_diff = oopspend - oopt_amt

* Generating bins for the data
gen icl_diff_widebin = 50*floor(icl_diff/50)
gen icl_diff_narrowbin = 20*floor(icl_diff/20)

* Generating the number of beneficiaries in each bin
bysort icl_diff_widebin: gen icl_totwide = _N
bysort icl_diff_narrowbin: gen icl_totnarrow = _N

* Generating an indicator for whether beneficiaries stopped claiming before December
foreach cut in any acute chr gen brand maint nonmaint hedis nonhedis {
	gen nocl_01dec_`cut' = month(date_`cut')<12
}

* Empirically calculating beneficiary cost sharing for each phase:
	* Step 1: Calculate the fraction of total cost in a phase paid OOP
	gen bene_coin_ded = ded_oopspend/ded_totcost
	gen bene_coin_preicl = preicl_oopspend/preicl_totcost
	gen bene_coin_gap = gap_oopspend/gap_totcost
	gen bene_coin_cat = cat_oopspend/cat_totcost

	* Eliminating infeasible amounts
	replace bene_coin_ded = 1 if bene_coin_ded>1 & !missing(bene_coin_ded)
	replace bene_coin_preicl = 1 if bene_coin_preicl>1 & !missing(bene_coin_preicl)
	replace bene_coin_gap = 1 if bene_coin_gap>1 & !missing(bene_coin_gap)
	replace bene_coin_cat = 1 if bene_coin_cat>1 & !missing(bene_coin_cat)

	* Step 2: Calculating the average beneficiary cost sharing as the 
	* 	  as the plan-year cost sharing
	bysort plan yr: egen plan_coin_ded = mean(bene_coin_ded)
	by plan yr: egen plan_coin_preicl = mean(bene_coin_preicl)
	by plan yr: egen plan_coin_gap = mean(bene_coin_gap)

	* Step 3: For plan-years and phases (e.g. catastrophic) where there are no or few
	* 	  observation, replace the plan-year average with a plan group-year average
	bysort plan_group yr: egen plan_group_coin_ded = mean(bene_coin_ded)
	by plan_group yr: egen plan_group_coin_preicl = mean(bene_coin_preicl)
	by plan_group yr: egen plan_group_coin_gap = mean(bene_coin_gap)
	by plan_group yr: egen plan_coin_cat = mean(bene_coin_cat)
	replace plan_coin_ded = plan_group_coin_ded if plan_coin_ded==. & plan_group=="ded"
	replace plan_coin_preicl = plan_group_coin_preicl if plan_coin_preicl==.
	replace plan_coin_gap = plan_group_coin_gap if plan_coin_gap==.

* Drop variable used for the 65 year old analysis but not the full sample
drop initial_total* days_to_first*


* Drop those without standard ICLs
drop if has_standardicl==0

* Merge in variables from externally generated datasets from Medicare Parts A and B
	* Part D risk scores (generated using CMS programs and Part A and B claims)
	merge 1:1 bene_id yr using mergefiles/RiskScoreOutputs/RiskScores_baseline, gen(rxhcc_merge) keep(match master)
	* 2006 Part D risk scores (generated using CMS programs and Part A and B claims)
	merge 1:1 bene_id yr using mergefiles/RiskScoreOutputs/RiskScores06_baseline, gen(rx06_merge) keep(match master)
	* Medicare Advantage risk scores (generated using CMS programs and Part A and B claims to predict Parts A and B spending)
	merge 1:1 bene_id yr using mergefiles/RiskScoreOutputs/MA_riskScores_baseline, gen(mahcc_merge) keep(match master)
	* Month-level utilization of Medicare Parts A and B
	merge 1:1 bene_id yr using mergefiles/MedicarePartsAandB/utilization_tomerge, gen(_merge_PartsAandB) keep(match master)
		* Recoding variables to zero for beneficiaries not matched
		foreach var of varlist op_admit* er_op_admit* ip_admit* er_ip_admit* {
			replace `var' = 0 if `var'==.
		}
	* Death dates from all available datasets 
	merge m:1 bene_id using mergefiles/MedicarePartsAandB/death_dates_full, gen(_merge_death) keep(match master)

* Saving out baseline analysis sample
save ${out}/bene070809_baseline, replace

} // if `baseline_restrictions' == 1

log close
