* CODING MEDICARE UTILIZATION * 

*Data libraries;
libname den2006 "/disk/agedisk2/medicare/data/20pct/den/2006";
libname den2007 "/disk/agedisk2/medicare/data/20pct/den/2007";
libname den2008 "/disk/agedisk2/medicare/data/20pct/den/2008";
libname ben2009 "/disk/agedisk2/medicare/data/20pct/bsf/2009";
libname ben2010 "/disk/agedisk2/medicare/data/20pct/bsf/2010";
%macro setlibs;
	%do year = 2006 %to 2010;
		libname car&year. "/disk/agedisk2/medicare/data/20pct/car/&year.";
		libname op&year. "/disk/agedisk2/medicare/data/20pct/op/&year.";
		libname ip&year. "/disk/agedisk2/medicare/data/20pct/ip/&year.";
		libname med&year. "/disk/agedisk2/medicare/data/20pct/med/&year.";
	%end;
%mend;
%setlibs;
libname output "/disk/agedisk3/medicare.work/finkels-DUA22559/tangb/contract_design/code/RiskScoreOutputs/MedicarePartsAandB";

%include "/disk/agedisk3/medicare.work/finkels-DUA22559/tangb/contract_design/code/CalculatingRiskScoresAndUtilization/SampleCreationMacros.sas";

* DEATH DATE FROM BENE/DEN FILES;
%macro GenerateDeathDates;

	%do year = 2006 %to 2010;

		%GenerateBeneYear(year=&year., pct=20);

		data benetemp;
			set benetemp;
			keep bene_id death_dt;
		run;

		data benedeath_&year.;
			set benetemp;
			year = &year.;
			rename death_dt = death_dt_bene;
		run;
		
		%GenerateMedClmsYear(year=&year., pct=20);

		data medtemp;
			set medtemp;
			keep bene_id deathdt;
		run;
		
		data medpardeath_&year.;
			set medtemp;
			rename deathdt = death_dt_medpar;
			year = &year.;
		run;
		
		proc sort data=medpardeath_&year.;
			by bene_id;
		run;
		proc sort data=benedeath_&year.;
			by bene_id;
		run;
		
		data output.death_&year.;
			merge benedeath_&year. medpardeath_&year.;
			by bene_id;
		run;
		
	%end;

%mend GenerateDeathDates;
%GenerateDeathDates;


* GENERATE UTILIZATION VARIABLES;
%macro GenerateUtilizationVariables;

	%do year = 2006 %to 2010;

		%GenerateMedClmsYear(year=&year., pct=20);

		data output.medpar_ipvisits&year.;
			set medtemp;
			keep bene_id admsndt er_amt;
		run;
		
		%GenerateOPRevCntrYear(year=&year, pct=20);
		%GenerateOpClmsYear(year=&year., pct=20);

		data oprevtemp;
			set oprevtemp;
			keep bene_id clm_id rev_cntr;
			ER=0;
		run;
		
		data optemp;
			set optemp;
			keep bene_id clm_id provider from_dt at_npi;
		run;

		data oprevtemp;
			set oprevtemp;
			if rev_cntr="0450" or rev_cntr="0451" or rev_cntr="0452" or rev_cntr="0453"
			or rev_cntr="0454" or rev_cntr="0455" or rev_cntr="0456" or rev_cntr="0457"
			or rev_cntr="0458" or rev_cntr="0459" or rev_cntr="0981" then ER=1;
			else ER=0;
		run;
		
		proc sort data=oprevtemp;
			by bene_id clm_id;
		run;
		
		proc means data=oprevtemp noprint;
			by bene_id clm_id;
			var ER;
			output out=oprevtemp2 max= ;
		run;
		
		proc sort data=optemp;
			by bene_id clm_id;
		run;
		proc sort data=oprevtemp2;
			by bene_id clm_id;
		run;
		
		data optemp2;
			merge optemp oprevtemp2;
			by bene_id clm_id;
		run;
		
		proc sort data=optemp2;
			by at_npi;
		run;
		proc sort data=output.specialist_list_for_merge;
			by at_npi;
		run;
		data optemp3;
			merge optemp2 output.specialist_list_for_merge;
			by at_npi;
		run;

		proc sort data=optemp3;
			by bene_id provider from_dt;
		run;
		
		proc means data=optemp3 noprint;
			by bene_id provider from_dt;
			var ER specialist;
			output out=output.opvisits&year. max= ;
		run;
		
		%GenerateIPRevCntrYear(year=&year, pct=20);
		%GenerateIpClmsYear(year=&year, pct=20);
		
		data iprevtemp;
			set iprevtemp;
			keep bene_id clm_id rev_cntr;
			ER=0;
		run;
		
		data iptemp;
			set iptemp;
			keep bene_id clm_id provider from_dt at_npi;
		run;

		data iprevtemp;
			set iprevtemp;
			if rev_cntr="0450" or rev_cntr="0451" or rev_cntr="0452" or rev_cntr="0453"
			or rev_cntr="0454" or rev_cntr="0455" or rev_cntr="0456" or rev_cntr="0457"
			or rev_cntr="0458" or rev_cntr="0459" or rev_cntr="0981" then ER=1;
			else ER=0;
		run;
		
		proc sort data=iprevtemp;
			by bene_id clm_id;
		run;
		
		proc means data=iprevtemp noprint;
			by bene_id clm_id;
			var ER;
			output out=iprevtemp2 max= ;
		run;
		
		proc sort data=iptemp;
			by bene_id clm_id;
		run;
		proc sort data=iprevtemp2;
			by bene_id clm_id;
		run;
		
		data iptemp2;
			merge iptemp iprevtemp2;
			by bene_id clm_id;
		run;
		
		proc sort data=iptemp2;
			by at_npi;
		run;
		proc sort data=output.specialist_list_for_merge;
			by at_npi;
		run;
		data iptemp3;
			merge iptemp2 output.specialist_list_for_merge;
			by at_npi;
		run;

		proc sort data=iptemp3;
			by bene_id provider from_dt;
		run;
		
		proc means data=iptemp3 noprint;
			by bene_id provider from_dt;
			var ER specialist;
			output out=output.ipvisits&year. max= ;
		run;
	%end;

%mend GenerateUtilizationVariables;
%GenerateUtilizationVariables;
