/* Version ID v3.12
Wed Jun 22 09:20:33 EDT 2011
*/
%macro decompress(spaceDir=,dataRoot=,ftype=,pct=,year=,subtype=,pctsuffix=);


    %include "&SASmac./AutoFileCountPipe.sas";
  * This macro navigates the Medicare claims data directory structure and decompresses;
    * the appropriate file to the requested location.  Adapted from code provided by Jean Roth.;
    * ;
    * spaceDir - the directory to which the file should be decompressed;
    * dataRoot - the root directory of the NBER claims data hierarchy;
    * ftype - the type of claims file (medpar, denom, car, op, ip, etc);
    * pct - the percentage extract to retrieve;
    * year - the year of the file to retrieve;
    ***********************************************************************************;
    * ***TURNED OF THIS FEATURE--DetermineMax MAcro Counts it now***********************;
            ***** min - For some years, claims data are split across multiple numbered files.;
            *****       This is the number of the first file;
            ***** max - this is the number of the last file;
    **************************************************************************;
    * subtype - used when there are two types of files for a given year;
    *           (e.g. claims and revenue center files);
    * pctsuffix - rarely-used option for when the file format includes some string after the extract percentage;

    *FOR COUTNING;

  %DetermineMax(datadir=&dataroot.,pct=&pct.,ftype=&ftype.,year=&year.,subtype=&subtype.,pctsuffix=&pctsuffix.);
  *NOTE THIS DATA STEP READS IN THE DATA FILE N which is OUTPUT FROM THE DETERMINEMAX MACRO AND SIMPLY CONTAINTS THE MAX NUMBER OF FILES;
    data _NULL_;
        set autocount;
        if _n_=1 then do;
            call symputx('maxmacro',n);
            stop;
            end;
    run;
    %put &maxmacro.;
    %do j=1 %to &maxmacro.;

        %let compFile=&dataRoot./&pct.pct/&ftype./&year./&ftype.&pct.&pctsuffix._&year._&subtype.&j..sas7bdat.bz2;
        %let decompFile=&spaceDir./&ftype.&pct.&pctsuffix._&year._&subtype.&j..sas7bdat;
        %if not %sysfunc(fileexist("&decompFile.")) %then %do;

            %let tBefore=%sysfunc(datetime());
            x "bzcat &compFile. > &decompFile.";
            %let tAfter=%sysfunc(datetime());

            %let elapsed=%sysfunc(round(%sysevalf(%sysevalf(%sysevalf(&tAfter.-&tBefore.)/60))));
            %put Decompressing file &ftype.&pct.&pctsuffix._&year._&subtype.&j. took &elapsed. minutes.;

         %end;
        %end;
    %mend;

%macro decompress_i(spaceDir=,dataRoot=,ftype=,pct=,year=,subtype=,pctsuffix=,indx=);

  * This macro navigates the Medicare claims data directory structure and decompresses;
    * the appropriate file to the requested location.  Adapted from code provided by Jean Roth.;
    * ;
    * spaceDir - the directory to which the file should be decompressed;
    * dataRoot - the root directory of the NBER claims data hierarchy;
    * ftype - the type of claims file (medpar, denom, car, op, ip, etc);
    * pct - the percentage extract to retrieve;
    * year - the year of the file to retrieve;
    ***********************************************************************************;
    * ***TURNED OF THIS FEATURE--DetermineMax MAcro Counts it now***********************;
            ***** min - For some years, claims data are split across multiple numbered files.;
            *****       This is the number of the first file;
            ***** max - this is the number of the last file;
    **************************************************************************;
    * subtype - used when there are two types of files for a given year;
    *           (e.g. claims and revenue center files);
    * pctsuffix - rarely-used option for when the file format includes some string after the extract percentage;

    *FOR COUTNING;
    *Note that 20% carrier files are made up of 4 %5 draws in year 98 - 05;
    %if &ftype.=car and (&year.>=1998 and &year.<=2005) and &pct.=20 %then %let pctm=05;
    %else %let pctm=&pct.;
        %let compFile=&dataRoot./&pct.pct/&ftype./&year./&ftype.&pctm.&pctsuffix._&year._&subtype.&indx..sas7bdat.bz2;
        %let decompFile=&spaceDir./&ftype.&pctm.&pctsuffix._&year._&subtype.&indx..sas7bdat;
        %if not %sysfunc(fileexist("&decompFile.")) %then %do;

            %let tBefore=%sysfunc(datetime());
            x "bzcat &compFile. > &decompFile.";
            %let tAfter=%sysfunc(datetime());

            %let elapsed=%sysfunc(round(%sysevalf(%sysevalf(%sysevalf(&tAfter.-&tBefore.)/60))));
            %put Decompressing file &ftype.&pct.&pctsuffix._&year._&subtype.&indx. took &elapsed. minutes.;

         %end;
    %mend;


%macro RepeatN(pfx,Nlow,Nhigh);
%* Generate a numbered list of symbols starting with the value of pfx;
%* Ex: if pfx=Hello, Nlow=1, and Nhigh=3, then the macro generates;
%* Hello1 Hello2 Hello3;

    %do i=&Nlow. %to &Nhigh.;
        &pfx.&i.
            %end;

    %mend;

%macro GenYM(pfx,Ylow,Yhigh);
%* Generate a list of variables for each month of the year starting in year;
%* Ylow and ending in year Yhigh in format <pfx>YYYYmM;

    %do y=&Ylow. %to &Yhigh.;
        %do m=1 %to 12;
            &pfx.&y.m&m.
                %end;
            %end;

    %mend;



*This a macro to draw a random sample of the data to work with the data ;
%macro getsample(fromdata =, idvar=, dataout=, srate=);
	proc sql;
	   create table _ids as
	   select distinct &idvar
	   from &fromdata;
	quit;

	* Select sample;
	proc surveyselect data = _ids method= SRS rep= 1 samprate= &srate
	seed = 12345 out = _sampled;
	run;

	* Get data;
	proc sql;
	   create table &dataout as
	   select *
	   from &fromdata
	   where &idvar in (select &idvar from _sampled);
	quit;
%mend getsample;

